%% Licensed Materials - Property of IBM
%% Restricted Materials of IBM.  Reproduction, modification and redistribution are prohibited.
%% IBM Rational Rhapsody Design Manager
%% © Copyright IBM Corporation 2013  
%% U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

function [bSuccess] = uploadSimulinkModelToDM(sysName, withRefModels, moveToAction, origSystem)
	path2mdl = which(sysName);
    if isempty(path2mdl)
        msg = strcat('Could not find path to model "', sysName, '"');
        disp(msg);
        bSuccess = false;
        return;
    end    
	pubDir = strcat(tempdir, 'SimulinkImporterData');
	[s,mess,messid] = mkdir(pubDir);

	bSuccess = 0;
  	sys = load_system(path2mdl);
	if isobject(sys) == 0
		tempSysName = get_param(sys,'name');
		
		sysName = tempSysName;
		pubDir = strcat(pubDir, '/',sysName);
		[s,mess,messid] = mkdir(pubDir);

		% Prepare java objects:
		publisher = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.Publisher', pubDir);
		if isjava(publisher) == 0 
			fprintf('ERROR: java object (Publisher) allocation failed\n');
			return;
		end
    
		helper = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.MscriptHelper');
		if isjava(helper) == 0 
			fprintf('ERROR: java object (MscriptHelper) allocation failed\n');
			return;
        end
        
        if strcmp(moveToAction, 'save') == 1
            publisher.setInSaveModel(1);
        end   
        
		publisher.setModelFile(path2mdl);
		publisher.createModel(sysName);
     
        % unlock library before collecting system data (needed for changing
        % the zoom factor to fit all  elements, and revert back to the
        % orignal zoom).
        libraryLocked = get_param(sysName, 'Lock');
        if strcmp(libraryLocked, 'on') == 1
            set_param(sysName, 'Lock', 'off');
        end 
            
        try
            getSystemData(sys, publisher, '', helper, pubDir, path2mdl, withRefModels, moveToAction, origSystem);
        catch
            return;
        end
        
        % restore the library lock
        if strcmp(libraryLocked, 'on') == 1
            set_param(sysName, 'Lock', 'on');
        end  
            
		% Serialize and move to DM
    	publisher.Serialize();   
        bSuccess = publisher.moveToDM();    
    end
end

function getSystemData(sys, publisher, parentSysPath, helper, publishDir, path2mdl, withRefModels, moveToAction, origSystem)
    try
        evalin('base', 'createDMConnection');
        dmConn = evalin('base', 'dmConnection');
    catch
        return;
    end	
    if isjava(dmConn) == 0
        return;
    end    
	    
    internalSysName = get_param(sys,'name');
    sysName = regexprep(internalSysName, '\n', ' ');
    sysDescript = get_param(sys,'Description');
    
    sysUriName = char(helper.legalizeMdlElemName(sysName));
    
    subSystem = 0;

    isLibrary = 0;
    try
        libraryType = get_param(sys, 'LibraryType');
        if strcmp(libraryType, 'BlockLibrary') == 1
            isLibrary = 1;
        end
    catch
    end
    
        
    if ~isempty(parentSysPath)
        simSystem = publisher.addSystem(sysName, sysUriName, sysDescript, 0, isLibrary);
        subSystem = 1;
    else
        simSystem = publisher.addSystem(sysName, sysUriName, sysDescript, 1, isLibrary);
    end
    
    legalParentSysPath = regexprep(parentSysPath, '\n', ' ');
    
    sysPath = sysName;
    if isempty(legalParentSysPath) == 0
        sysPath = strcat(legalParentSysPath,'/',sysName);
    end
        
	% copy the model file in the publishDir
    if subSystem == 0
        copyfile(path2mdl, publishDir);
    end
    
    % copy the 'slblocks.m' is case we are a library
    if isLibrary == 1
        slblocksLocation = char(dmConn.getLibrarySlblocksPath());
        if isempty(slblocksLocation) == 0
            copyfile(slblocksLocation, publishDir);
        end    
    end    
      
    minXY = getMinXYOfSimulinkModel(sys, parentSysPath);
    if isempty(minXY) == 0
        simSystem.setMinXY(minXY(1), minXY(2));
    end    
    
    % Print diagram:
    % ======================
    stateFlowType = '';
    try
        stateFlowType = get_param(sysPath, 'SFBlockType');
    catch
  %      fprintf('No SFBlockType found for %s\n', sysPath);
    end
    
    if strcmp(stateFlowType, 'Chart') == 1
        printChartDiagramOld(sys, sysPath, publishDir, helper, origSystem, simSystem);
    elseif isempty(stateFlowType) == 1 || strcmp(stateFlowType, 'NONE') == 1
        printSystemDiagram(sys, publishDir, helper, origSystem, simSystem);
    end
    
    % Get system lines:
    % ======================
    SysLines = get_param (sys,'Lines');
    numOfLines = size(SysLines);
    
    for z = 1:numOfLines
        lineName = SysLines(z).Name;
        srcBlock = get_param(SysLines(z).SrcBlock, 'Name');
        legalSrcBlock = char(helper.legalizeMdlElemName(srcBlock));
        destBlock = get_param(SysLines(z).DstBlock, 'Name');
        legalDestBlock = char(helper.legalizeMdlElemName(destBlock));
        
        publisher.addSignal(lineName, legalSrcBlock, legalDestBlock);
    end
    
    % Get system blocks:
    % ======================
    blocks = get_param (sys,'Blocks');
    numOfBlocks = size(blocks);

    for x = 1:numOfBlocks
        
        tempBlockName = blocks{x};
        internalBlockName = regexprep(tempBlockName, '/', '//');
        legalBlockName = regexprep(internalBlockName, '\n', ' ');
        tempBlockPath = strcat(sysPath,'/',legalBlockName);
        internalBlockPath = horzcat(sysPath, '/',legalBlockName);
        blockPath = regexprep(tempBlockPath, '\n', ' ');
        
        if subSystem == 1
            legalSysPath = char(helper.legalizeMdlElemName(sysPath));
            legalBlockName2 = char(helper.legalizeMdlElemName(legalBlockName));
            uriName = strcat(legalSysPath, '_subsys_', legalBlockName2);
        else
            uriName = char(helper.legalizeMdlElemName(blockPath));
        end
        
        blockPosition = get_param (internalBlockPath,'Position');
        blockType = get_param (internalBlockPath,'BlockType');
       
        referenceBlock = get_param(internalBlockPath, 'ReferenceBlock'); 
        if isempty(referenceBlock) == 0
            blockType = strrep(referenceBlock, '/', '_');
        end
        
        if strcmp(blockType, 'SubSystem') == 1
            try
                SFBlockType = get_param(internalBlockPath, 'SFBlockType');
                if strcmp(SFBlockType, 'MATLAB Function') == 1
                    blockType = 'MatlabFunction';
                end
            catch 
            end    
        end
        
        blockDesc = get_param (internalBlockPath,'Description');
        
        dblArray = javaArray ('java.lang.Double', 4);

        dblArray(1) = java.lang.Double(blockPosition(1));
        dblArray(2) = java.lang.Double(blockPosition(2));
        dblArray(3) = java.lang.Double(blockPosition(3));
        dblArray(4) = java.lang.Double(blockPosition(4));
        
        simBlock = publisher.addBlock(sysUriName, legalBlockName, uriName, blockType, blockDesc, dblArray);
        
        if isempty(referenceBlock) == 0
            refBlockName = referenceBlock;
            refBlockName = regexprep(refBlockName, '\n', ' ');
            refBlockUriName = char(helper.legalizeMdlElemName(refBlockName));
            publisher.addReferenceBlock(sysUriName, refBlockName, refBlockUriName);
        end
        
        ObjParamsStruct = get_param(internalBlockPath, 'DialogParameters');
        numOfParams = 0;
        try
            if isempty(ObjParamsStruct) == 0
                ObjParams = struct2cell(ObjParamsStruct);
                InternalNames = fieldnames(ObjParamsStruct);
                numOfParams = size(ObjParams);
            end    
        catch
            fprintf('ERROR: struct2cell() failure\n');
        end
         
        getProps = 1;
        if strcmp(blockType, 'Scope') == 1
            getProps = 0;
        elseif numOfParams < 1
            getProps = 0;
        end
        
        % Get block parameters:
        % ======================
        if getProps > 0
            for y = 1:numOfParams                
                paramName = ObjParams{y}.Prompt;                
                paramVal = get_param(internalBlockPath, InternalNames{y});
                if ischar(paramName) && ischar(paramVal)
                    publisher.addProperty(paramName, paramVal);
                end
            end
        end
        
        % get sub system data:
        % ======================
        if strcmp(blockType, 'SubSystem') == 1
            subsys = find_system(sys, 'name', internalBlockName);
            if isobject(subsys) == 0 && isempty(subsys) == 0
          %      openFunc = get_param(subsys, 'OpenFcn');
          %      if isempty(openFunc)
                    try
                        subsys = subsys(1);
                        
                        stateFlowType = '';
                        try
                            stateFlowType = get_param(internalBlockPath, 'SFBlockType');
                        catch
                        end
    
                        if strcmp(stateFlowType, 'Chart') == 1
                            simBlock.setType('Chart');
                            getStateflowData(internalBlockPath, publisher, sysPath, helper, publishDir, sys, '');
                        else    
                            getSystemData(subsys, publisher, sysPath, helper, publishDir, path2mdl, withRefModels, moveToAction, sys);
                        end    
                        open_system(sys);
                    catch e
                        disp(e);
                    end    
           %     end
            end
        end
        
        %get Reference Model data:
        % ======================
        if strcmp(blockType, 'ModelReference') == 1
            refMdlName = get_param(internalBlockPath, 'ModelName');
            refSysName = regexprep(refMdlName, '\n', ' ');
            refSysUriName = char(helper.legalizeMdlElemName(refSysName));
            publisher.addReferenceSystem(refSysName, refSysUriName);
            
            if strcmp(withRefModels, 'include') == 1 || strcmp(withRefModels, 'includeRecursive') == 1
                if dmConn.doesSimulinkModelExistOnDM(refSysName) == 1
                    msg = sprintf('Model "%s" already exist in DM, skiping...',refSysName); 
                    disp(msg);
                else    
                    if strcmp(moveToAction, 'publish') == 1
                        msg = sprintf('Publishing reference model "%s" to DM...',refSysName); 
                    else    
                        msg = sprintf('Moving reference model "%s" to DM...',refSysName); 
                    end    
                    disp(msg);
              
                    if strcmp(withRefModels, 'include') == 1
                        withRefModels = 'exclude';
                    end
                
                    bSuccess = uploadSimulinkModelToDM(refSysName, withRefModels, moveToAction, origSystem);
                    if bSuccess == 1
                        if strcmp(moveToAction, 'publish') == 1
                            disp('Publish reference model done');
                        else    
                            set_param(refSysName,'PostSaveFcn','postOpenFromDMActions()');
                            save_system(refSysName);
			
                            disp('Move reference model done.');
		
                            %last, lock the model since save/moveto release the locking
                            if strcmp(moveToAction, 'MoveToAndLock') == 1
                                dmConn.lockSimulinkModel(refSysName);
                            end
                        end    
                        close_system(refSysName);
                    else
                        if strcmp(moveToAction, 'publish') == 1
                            disp('Failed to publish reference model to DM');
                        else    
                            disp('Failed to move reference model to DM');
                        end
                    end
                end
            else
                % if ref model doesn't exist in the current Project Area
                % ask the user to provide the Project Area
                if dmConn.doesSimulinkModelExistOnDM(refSysName) == 0
                    refSysUri = char(dmConn.getSimulinkRefModelUri(refSysName));
                    if isempty(refSysUri) == 0
                        projectArea = char(dmConn.getProjectAreaOfSimulinkModel(refSysName, refSysUri));
                    else    
                        projectArea = char(dmConn.openSelectRefModelProjectAreaDialog(refSysName));
                        if isempty(projectArea) == 1
                            ex = MException();
                            throw(ex);
                        end   
                    end    
                    simBlock.setRefModelProjectArea(projectArea);
                end    
            end
        end
    end
end
    
function printSystemDiagram(sys, publishDir, helper, origSystem, simSystem)
    orient(sys,'landscape');

    internalSysName = get_param(sys,'name');
    tempSysUriName = strrep(internalSysName, '/', '_');
    sysUriName = char(helper.legalizeMdlElemName(tempSysUriName));
    diagPicDir = strcat(publishDir,'/', sysUriName ,'.bmp');
    
    handle = get_param(sys, 'Handle');
    origModelHandle = get_param(origSystem, 'Handle');
    try
        % the model must be opened before saveas bitmap
        if handle ~= origModelHandle 
            mask = '';
            try
                mask = get_param(handle, 'mask');
            catch
            end    
            if strcmp(mask, 'on') == 1
                open_system(handle, 'force');
            else
                open_system(handle);
            end    
        end
        
        % turn off selection before saving the diagram bitmap
        selectedHandle = find_system(sys, 'SearchDepth', 1, 'FindAll', 'on','Type', 'Block', 'Selected', 'on');
        if isempty(selectedHandle) == 0
            set_param(selectedHandle, 'Selected', 'off');
        end
         
        % make sure all elements fit in the diagram 
        zoomFactor = get_param(sys, 'ZoomFactor');
        set_param(sys, 'ZoomFactor', 'FitSystem');
        newZoomFactor = get_param(sys, 'ZoomFactor');
        simSystem.setZoomFactor(newZoomFactor);
        
        saveas(handle, diagPicDir);
        
        % restore selection
        if isempty(selectedHandle) == 0
            set_param(selectedHandle, 'Selected', 'on');
        end
        
        % restore old zoom
        set_param(sys, 'ZoomFactor', zoomFactor);
        
        % restore the old model if needed
       % if handle ~= origModelHandle  
       %     open_system(origModelHandle);
       % end
    catch e
        disp(e);
        close_system(handle);
        open_system(origModelHandle);
    end    
end
     
function printChartDiagramOld(sys, sysPath, publishDir, helper, origSystem, simSystem)
    orient(sys,'landscape');

    internalSysName = get_param(sys,'name');
    tempSysUriName = strrep(internalSysName, '/', '_');
    sysUriName = char(helper.legalizeMdlElemName(tempSysUriName));
    diagPicDir = strcat(publishDir,'/', sysUriName ,'.bmp');
    
    handle = get_param(sys, 'Handle');
    origModelHandle = get_param(origSystem, 'Handle');
    try
        % the model must be opened before saveas bitmap
        if handle ~= origModelHandle       
            open_system(handle);
        end
        
        % make sure all elements fit in the diagram 
        zoomFactor = get_param(sys, 'ZoomFactor');
        set_param(sys, 'ZoomFactor', 'FitSystem');
        newZoomFactor = get_param(sys, 'ZoomFactor');
        simSystem.setZoomFactor(newZoomFactor);
        
        sfprint(sysPath, 'bitmap', diagPicDir, 0);
     
        % restore old zoom
        set_param(sys, 'ZoomFactor', zoomFactor);
        
    catch e
        disp(e); 
        close_system(handle);
        open_system(origModelHandle);
    end    
end

function getStateflowData(chart, publisher, parentSysPath, helper, publishDir, origSystem, graphicalSimSystem)
    if isempty(chart) == 1
        return;
    end   
    
    if isa(chart, 'Stateflow.State') == 1 || isa(chart, 'Stateflow.Box') == 1 || isa(chart, 'Stateflow.Function') == 1 || isa(chart, 'Stateflow.Chart') == 1
        ch = chart;
        chartName = chart.Name;
    else    
        chartName = get_param(chart, 'Name');
        root = sfroot;
        machine = root.find('-isa','Simulink.BlockDiagram');
        ch = machine.find('-isa','Stateflow.Chart', '-and', 'Name', chartName);
    end
        
    if isempty(ch) == 1
        return;
    end
    
    chartName = regexprep(chartName, '\n', ' '); 
    chartUriName = char(helper.legalizeMdlElemName(chartName));
    if ~isempty(parentSysPath)
        simSystem = publisher.addSystem(chartName, chartUriName, ch.Description, 0, 0);
        subSystem = 1;
    else
        simSystem = publisher.addSystem(chartName, chartUriName, ch.Description, 1, 0);
    end
    
    legalParentSysPath = regexprep(parentSysPath, '\n', ' '); 
    chartPath = chartName;
    if isempty(legalParentSysPath) == 0
        chartPath = strcat(legalParentSysPath,'/',chartName);
    end
   
    hasGraphics = 0;
    if isa(ch, 'Stateflow.Chart') == 1 || ch.IsSubchart == 1
        hasGraphics = 1;
        graphicalSimSystem = simSystem;
        minXY = getMinXYOfStateFlowChart(ch);
        if isempty(minXY) == 0
            simSystem.setMinXY(minXY(1), minXY(2));
        end    
    
        % Print diagram
        printChartDiagram(ch, chartPath, publishDir, helper, origSystem, simSystem);
    end
    
    % next, get all top level objects of the chart (i.e. States, Boxes, Function etc)
    objs = ch.find('-depth', 1);
    objs = setdiff(objs, ch);
    numOfObjs = size(objs);
    for i = 1 : numOfObjs
        obj = objs(i);
        if isa(obj, 'Stateflow.Data') == 1 || isa(obj, 'Stateflow.Event') == 1 || isa(obj, 'Stateflow.Transition')
            continue;
        end
        
        if isa(obj, 'Stateflow.Junction')
            name = sprintf('%d', obj.SSIdNumber);
            if strcmp(obj.Type,'HISTORY') == 1
                type = 'HistoryJunction';
            else
                type = 'ConnectiveJunction';
            end
            x1 = obj.Position.Center(1) - obj.Position.Radius;
            y1 = obj.Position.Center(2) - obj.Position.Radius;
            x2 = obj.Position.Center(1) + obj.Position.Radius;
            y2 = obj.Position.Center(2) + obj.Position.Radius;
            position = [x1 y1 x2 y2];
        else
            name = obj.Name;
            if isa(obj, 'Stateflow.State') == 1
                type = 'State';
            elseif isa(obj, 'Stateflow.Box') == 1   
                type = 'Box';
            elseif isa(obj, 'Stateflow.Function') == 1
                type = 'Function';
            elseif isa(obj, 'Stateflow.EMFunction') == 1
                type = 'MatlabFunction';
            elseif isa(obj, 'Stateflow.TruthTable') == 1
                type = 'TruthTable'; 
            elseif isa(obj, 'Stateflow.Note') == 1
                type = 'Annotation';
            end
            position = obj.Position;
            position(3) = position(1) + position(3);
            position(4) = position(2) + position(4);
        end   
        
        internalBlockName = regexprep(name, '/', '//');
        legalBlockName = regexprep(internalBlockName, '\n', ' ');
        tempBlockPath = strcat(chartPath,'/',legalBlockName);
        internalBlockPath = horzcat(chartPath, '/',legalBlockName);
        blockPath = regexprep(tempBlockPath, '\n', ' ');
        
        if subSystem == 1
            legalSysPath = char(helper.legalizeMdlElemName(chartPath));
            legalBlockName2 = char(helper.legalizeMdlElemName(legalBlockName));
            uriName = strcat(legalSysPath, '_subsys_', legalBlockName2);
        else
            uriName = char(helper.legalizeMdlElemName(blockPath));
        end
        
        objDesc = obj.Description;
        
        dblArray = javaArray ('java.lang.Double', 4);

        dblArray(1) = java.lang.Double(position(1));
        dblArray(2) = java.lang.Double(position(2));
        dblArray(3) = java.lang.Double(position(3));
        dblArray(4) = java.lang.Double(position(4));
        
        simBlock = publisher.addBlock(chartUriName, legalBlockName, uriName, type, objDesc, dblArray);
        if hasGraphics == 0 && isempty(graphicalSimSystem) == 0
            graphicalSimSystem.addGraphicalBlock(simBlock);
        end    
        % now, check is this object has inner object (only in State, Box
        % and function)
        if isa(obj, 'Stateflow.Box') == 1 || isa(obj, 'Stateflow.Function') == 1 || isa(obj, 'Stateflow.State') == 1
            innerObjs = obj.find('-depth', 1);
            innerObjs = setdiff(innerObjs, obj);
            if size(innerObjs) > 0
                getStateflowData(obj, publisher, chartPath, helper, publishDir, origSystem, graphicalSimSystem);
            end 
            if obj.IsSubchart == 1
                obj.view;
            end    
        end    
    end    
end

function printChartDiagram(chart, chartPath, publishDir, helper, origSystem, simSystem)
    if isempty(chart) == 1
        return;
    end
    
    if isa(chart, 'Stateflow.State') == 0 && isa(chart, 'Stateflow.Box') == 0 && ...
        isa(chart, 'Stateflow.Function') == 0 && isa(chart, 'Stateflow.Chart') == 0
        return
    end
    
    if isa(chart, 'Stateflow.Box') == 1 || isa(chart, 'Stateflow.Function') == 1 || isa(chart, 'Stateflow.State') == 1
        baseChart = chart.chart;
    else
        baseChart = chart;
    end
    
    % make sure the diagram is opened
    chart.view;
    
    internalSysName = chart.Name;
    tempSysUriName = strrep(internalSysName, '/', '_');
    sysUriName = char(helper.legalizeMdlElemName(tempSysUriName));
    diagPicDir = strcat(publishDir,'/', sysUriName ,'.bmp');
    
    try
         
        % make sure all elements fit in the diagram 
        zoomFactor = baseChart.Editor.ZoomFactor;
        chart.fitToView;
        newZoomFactor = baseChart.Editor.ZoomFactor;
        if newZoomFactor < 10
            newZoomFactor = newZoomFactor * 100;
        end    
        simSystem.setZoomFactor(newZoomFactor);
        
        sfprint(chart, 'bitmap', diagPicDir, 0);
     
        % restore old zoom
		if baseChart.Editor.ZoomFactor ~= zoomFactor && baseChart.Locked == 0 && baseChart.Iced == 0
			baseChart.Editor.ZoomFactor = zoomFactor; 
		end	
        
    catch e
        disp(e); 
        if isempty(origSystem) == 0
            if isa(chart, 'Stateflow.State') == 1 || isa(chart, 'Stateflow.Box') == 1 || isa(chart, 'Stateflow.Function') == 1 || isa(chart, 'Stateflow.Chart') == 1
                origSystem.view;
            else
                open_system(origSystem);
            end    
        end    
    end    
end